/*
 * AVR based RGB Moodlight
 *
 * moodlight.c - moodlight modes and remote control
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: moodlight.c 145 2012-01-11 21:40:58Z schwenke $
 */

#include <inttypes.h>

#include "irmp.h"
#include "moodlight.h"


/* remote control address and command codes */

#define MY_REMOTE   0x916e
#define KEY_OPERATE 0x14
#define KEY_1       0x05
#define KEY_2       0x06
#define KEY_3       0x07
#define KEY_4       0x0c
#define KEY_5       0x0d
#define KEY_6       0x0e
#define KEY_7       0x0f
#define KEY_8       0x1c
#define KEY_9       0x1d
#define KEY_0       0x04
#define KEY_MINUS   0x18
#define KEY_PLUS    0x19
#define KEY_REV     0x02
#define KEY_FWD     0x03
#define KEY_STOP    0x01
#define KEY_PLAY    0x08
#define KEY_REC     0x09
#define KEY_PAUSE   0x0b


/* global moodlight variables(s) */

struct {
    uint8_t  mode;
    uint8_t  speed;
    uint16_t cycle;
    uint16_t color;
} ml;

/* led brightness (0..MAX_BRIGHT) */

uint8_t level_r;
uint8_t level_g;
uint8_t level_b;

/* clock tick, decremented once in each PWM cycle */

volatile uint8_t tick;

/* last received IR signal */

IRMP_DATA irmp_data;


/* evaluate an IR remote signal */
void moodlight_remote(void)
{
    if (   irmp_data.protocol != IRMP_NEC_PROTOCOL
        || irmp_data.address  != MY_REMOTE) {
        return;
    }

    switch (irmp_data.command) {

    case KEY_OPERATE: /* on/off */
        {
            if ((irmp_data.flags & IRMP_FLAG_REPETITION) == 0) {
                ml.mode ^= 0x80;
            }
            if (ml.mode & 0x80) {
                level_r = 0;
                level_g = 0;
                level_b = 0;
            }
            break;
        }

    case KEY_1:
        {
            ml.mode = 0;
            break;
        }

    case KEY_2:
        {
            ml.mode = 1;
            break;
        }

    case KEY_FWD:
        {
            if ((ml.mode == 0 || ml.mode == 1) && ml.speed > 1) {
                ml.speed--;
            }
            break;
        }

    case KEY_REV:
        {
            if ((ml.mode == 0 || ml.mode == 1) && ml.speed < 16) {
                ml.speed++;
            }
            break;
        }

    default:
        {
            break;
        }
    }
}


/* initialization */
void moodlight_init(void)
{
    ml.mode  = 1;
    ml.speed = 6;
    ml.cycle = 1;
    tick = 1;
}


/* called after last pwm phase when tick == 0 */
void moodlight_step(void)
{
    switch (ml.mode) {

    case 0: /* rotation R-G-B */
    case 1: /* rotation R-B-G */
        {
            if (ml.cycle <= MAX_BRIGHT) {
                level_r = (MAX_BRIGHT+1) - ml.cycle;
                level_g = ml.cycle;
                level_b = 0;
            }
            else if (ml.cycle <= 2*MAX_BRIGHT) {
                level_r = 0;
                level_g = (2*MAX_BRIGHT+1) - ml.cycle;
                level_b = ml.cycle - MAX_BRIGHT;
            }
            else {
                level_r = ml.cycle - (2*MAX_BRIGHT);
                level_g = 0;
                level_b = (3*MAX_BRIGHT+1) - ml.cycle;
            }

            if (ml.mode == 0) {
                ml.cycle ++;
                if (ml.cycle == (3*MAX_BRIGHT+1)) {
                    ml.cycle = 1;
                }
            } else {
                ml.cycle --;
                if (ml.cycle == 0) {
                    ml.cycle = (3*MAX_BRIGHT);
                }
            }
            tick = ml.speed;

            break;
        }


    }
}

